@@ -1,27 +1,29 @@
 """ RemoveComprehension turns list comprehension into function calls. """
 
-from pythran import metadata
 from pythran.analyses import ImportedIds
 from pythran.passmanager import Transformation
 
-import ast
+import pythran.metadata as metadata
+
+import gast as ast
+from functools import reduce
 
 
 class RemoveComprehension(Transformation):
     """
     Turns all list comprehension from a node into new function calls.
 
-    >>> import ast
+    >>> import gast as ast
     >>> from pythran import passmanager, backend
     >>> node = ast.parse("[x*x for x in (1,2,3)]")
     >>> pm = passmanager.PassManager("test")
     >>> _, node = pm.apply(RemoveComprehension, node)
-    >>> print pm.dump(backend.Python, node)
+    >>> print(pm.dump(backend.Python, node))
     list_comprehension0()
     def list_comprehension0():
-        __target = __builtin__.list()
+        __target = builtins.list()
         for x in (1, 2, 3):
-            __builtin__.list.append(__target, (x * x))
+            builtins.list.append(__target, (x * x))
         return __target
     """
 
@@ -39,16 +41,16 @@ class RemoveComprehension(Transformation):
 
         Examples
         --------
-        >> [i for i in xrange(2)]
+        >> [i for i in range(2)]
 
         Becomes
 
-        >> for i in xrange(2):
+        >> for i in range(2):
         >>    ... x code with if clauses ...
 
         It is a reducer as it can be call recursively for mutli generator.
 
-        Ex : >> [i, j for i in xrange(2) for j in xrange(4)]
+        Ex : >> [i, j for i in range(2) for j in range(4)]
         """
         def wrap_in_ifs(node, ifs):
             """
@@ -56,11 +58,11 @@ class RemoveComprehension(Transformation):
 
             Examples
             --------
-            >> [i for i in xrange(2) if i < 3 if 0 < i]
+            >> [i for i in range(2) if i < 3 if 0 < i]
 
             Becomes
 
-            >> for i in xrange(2):
+            >> for i in range(2):
             >>    if i < 3:
             >>        if 0 < i:
             >>            ... the code from `node` ...
@@ -68,13 +70,14 @@ class RemoveComprehension(Transformation):
             Note the nested ifs clauses.
             """
             return reduce(lambda n, if_: ast.If(if_, [n], []), ifs, node)
-        return ast.For(g.target, g.iter, [wrap_in_ifs(x, g.ifs)], [])
+        return ast.For(g.target, g.iter, [wrap_in_ifs(x, g.ifs)], [], None)
 
     def visit_AnyComp(self, node, comp_type, *path):
+        self.update = True
         node.elt = self.visit(node.elt)
         name = "{0}_comprehension{1}".format(comp_type, self.count)
         self.count += 1
-        args = self.passmanager.gather(ImportedIds, node, self.ctx)
+        args = self.gather(ImportedIds, node)
         self.count_iter = 0
 
         starget = "__target"
@@ -85,47 +88,46 @@ class RemoveComprehension(Transformation):
                               reduce(lambda x, y: ast.Attribute(x, y,
                                                                 ast.Load()),
                                      path[1:],
-                                     ast.Name(path[0], ast.Load())),
-                              [ast.Name(starget, ast.Load()), node.elt],
+                                     ast.Name(path[0], ast.Load(),
+                                              None, None)),
+                              [ast.Name(starget, ast.Load(), None, None),
+                               node.elt],
                               [],
-                              None,
-                              None
                               )
                           )
                       )
         # add extra metadata to this node
         metadata.add(body, metadata.Comprehension(starget))
         init = ast.Assign(
-            [ast.Name(starget, ast.Store())],
+            [ast.Name(starget, ast.Store(), None, None)],
             ast.Call(
                 ast.Attribute(
-                    ast.Name('__builtin__', ast.Load()),
+                    ast.Name('builtins', ast.Load(), None, None),
                     comp_type,
                     ast.Load()
                     ),
-                [], [], None, None)
+                [], [],)
             )
-        result = ast.Return(ast.Name(starget, ast.Load()))
-        sargs = sorted(ast.Name(arg, ast.Param()) for arg in args)
+        result = ast.Return(ast.Name(starget, ast.Load(), None, None))
+        sargs = [ast.Name(arg, ast.Param(), None, None) for arg in args]
         fd = ast.FunctionDef(name,
-                             ast.arguments(sargs, None, None, []),
+                             ast.arguments(sargs, [], None, [], [], None, []),
                              [init, body, result],
-                             [])
+                             [], None, None)
+        metadata.add(fd, metadata.Local())
         self.ctx.module.body.append(fd)
         return ast.Call(
-            ast.Name(name, ast.Load()),
-            [ast.Name(arg.id, ast.Load()) for arg in sargs],
+            ast.Name(name, ast.Load(), None, None),
+            [ast.Name(arg.id, ast.Load(), None, None) for arg in sargs],
             [],
-            None,
-            None
             )  # no sharing !
 
     def visit_ListComp(self, node):
         return self.visit_AnyComp(node, "list",
-                                  "__builtin__", "list", "append")
+                                  "builtins", "list", "append")
 
     def visit_SetComp(self, node):
-        return self.visit_AnyComp(node, "set", "__builtin__", "set", "add")
+        return self.visit_AnyComp(node, "set", "builtins", "set", "add")
 
     def visit_DictComp(self, node):
         # this is a quickfix to match visit_AnyComp signature
@@ -137,10 +139,11 @@ class RemoveComprehension(Transformation):
         return self.visit_AnyComp(node, "dict", "__dispatch__", "update")
 
     def visit_GeneratorExp(self, node):
+        self.update = True
         node.elt = self.visit(node.elt)
         name = "generator_expression{0}".format(self.count)
         self.count += 1
-        args = self.passmanager.gather(ImportedIds, node, self.ctx)
+        args = self.gather(ImportedIds, node)
         self.count_iter = 0
 
         body = reduce(self.nest_reducer,
@@ -148,16 +151,14 @@ class RemoveComprehension(Transformation):
                       ast.Expr(ast.Yield(node.elt))
                       )
 
-        sargs = sorted(ast.Name(arg, ast.Param()) for arg in args)
+        sargs = [ast.Name(arg, ast.Param(), None, None) for arg in args]
         fd = ast.FunctionDef(name,
-                             ast.arguments(sargs, None, None, []),
-                             [body],
-                             [])
+                             ast.arguments(sargs, [], None, [], [], None, []),
+                             [body], [], None, None)
+        metadata.add(fd, metadata.Local())
         self.ctx.module.body.append(fd)
         return ast.Call(
-            ast.Name(name, ast.Load()),
-            [ast.Name(arg.id, ast.Load()) for arg in sargs],
+            ast.Name(name, ast.Load(), None, None),
+            [ast.Name(arg.id, ast.Load(), None, None) for arg in sargs],
             [],
-            None,
-            None
             )  # no sharing !
